"use strict";

var assert = require("assert");

var adapter = global.adapter;
var fulfilled = adapter.fulfilled;
var rejected = adapter.rejected;
var pending = adapter.pending;

describe("[Extension] Promises are always resolved asynchronously", function () {
    it("should be asynchronous for already-fulfilled promises", function (done) {
        var thenHasReturned = false;

        fulfilled().then(function () {
            assert(thenHasReturned);
            done();
        });

        thenHasReturned = true;
    });

    it("should be asynchronous for already-rejected promises", function (done) {
        var thenHasReturned = false;

        rejected().then(null, function () {
            assert(thenHasReturned);
            done();
        });

        thenHasReturned = true;
    });

    it("should be asynchronous for eventually-fulfilled promises", function (done) {
        var thenHasReturned = false;

        var tuple = pending();
        tuple.promise.then(function () {
            assert(thenHasReturned);
            done();
        });

        thenHasReturned = true;
        tuple.fulfill();
    });

    it("should be asynchronous for eventually-rejected promises", function (done) {
        var thenHasReturned = false;

        var tuple = pending();
        tuple.promise.then(null, function () {
            assert(thenHasReturned);
            done();
        });

        thenHasReturned = true;
        tuple.reject();
    });
});
